﻿namespace SampleApp
{
    using Infragistics.Win.UltraWinGrid;
    using System;
    using System.Drawing;
    using Infragistics.Win;
    using System.Collections.Generic;
    using System.Windows.Forms;

    public partial class Form1 : Form
    {
        private int valueCellWidth = 0;
        private int valueCellHeight = 0;

        public Form1()
        {
            InitializeComponent();
        }


        private void Form1_Load(object sender, EventArgs e)
        {

        }

        private void ultraGrid1_InitializeLayout(object sender, InitializeLayoutEventArgs e)
        {
            UltraGridLayout layout = e.Layout;
            UltraGridBand rootBand = layout.Bands[0];

            rootBand.CardView = true;            
            rootBand.RowLayoutStyle = RowLayoutStyle.ColumnLayout; 
        }

        private void button1_Click(object sender, System.EventArgs e)
        {
            List<Person> ls = new List<Person>();
            ls.Add(new Person("x", "y" + Environment.NewLine + "z" + Environment.NewLine + "t"));
            ls.Add(new Person("1", "2"));

            ultraGrid1.DataSource = ls;
            this.ultraGrid1.DisplayLayout.Bands[0].Columns["Value"].CellMultiLine = DefaultableBoolean.True;
            this.ultraGrid1.DisplayLayout.Bands[0].CardSettings.AllowSizing = true;
        }

        private void ultraGrid1_InitializeRow(object sender, InitializeRowEventArgs e)
        {
            var grid = sender as UltraGrid;
            var rootBand = grid.DisplayLayout.Bands[0];
            var cell = e.Row.Cells["Value"];

            using (Graphics graphics = CreateGraphics())
            {
                var cellTextSize = graphics.MeasureString(cell.Text, grid.Font);
                var cellTextHeight = (int)cellTextSize.Height;
                var cellTextWidth = (int)cellTextSize.Width;

                valueCellHeight = cellTextHeight > valueCellHeight ? cellTextHeight : valueCellHeight;
                valueCellWidth = cellTextWidth > valueCellWidth ? cellTextWidth : valueCellWidth;
            }

            rootBand.Columns["Value"].RowLayoutColumnInfo.PreferredCellSize = new Size(valueCellWidth + 5, valueCellHeight + 5);
        }

        private void ultraGrid1_CellChange(object sender, CellEventArgs e)
        {
            var grid = sender as UltraGrid;
            var rootBand = grid.DisplayLayout.Bands[0];
            var column = e.Cell.Column;
            var originalCellValue = e.Cell.OriginalValue.ToString();
            var newCellValue = e.Cell.Text.ToString();

            if (column.Key == "Value")
            {
                valueCellHeight = 0;
                valueCellWidth = 0;

                using (Graphics graphics = CreateGraphics())
                {
                    var newValueTextSize = graphics.MeasureString(newCellValue, grid.Font);
                    var newValueTextHeight = (int)newValueTextSize.Height;
                    var newValueTextWidth = (int)newValueTextSize.Width;

                    valueCellHeight = newValueTextHeight > valueCellHeight ? newValueTextHeight : valueCellHeight;
                    valueCellWidth = newValueTextWidth > valueCellWidth ? newValueTextWidth : valueCellWidth;
                }

                rootBand.Columns[column.Key].RowLayoutColumnInfo.PreferredCellSize = new Size(valueCellWidth + 5, valueCellHeight + 5);
            }
        }


        private void ultraGrid1_AfterRowLayoutItemResized(object sender, AfterRowLayoutItemResizedEventArgs e)
        {           
            var grid = sender as UltraGrid;
            var rootBand = grid.DisplayLayout.Bands[0];
            var rows = grid.DisplayLayout.Rows;

            if (e.RowLayoutItem.GetType().Name == "UltraGridColumn")
            {
                var column = e.RowLayoutItem as UltraGridColumn;

                if (column.Key == "Value" && e.RowLayoutItem.PreferredSize.Width != valueCellWidth)
                {
                    valueCellWidth = e.RowLayoutItem.PreferredSize.Width;
                    valueCellHeight = 0;

                    foreach (var row in rows)
                    {
                        var cellText = row.Cells["Value"].Text;
                        using (Graphics graphics = CreateGraphics())
                        {
                            var cellTextSize = graphics.MeasureString(cellText, grid.Font, valueCellWidth);
                            var textHeight = (int)cellTextSize.Height;
                            valueCellHeight = textHeight > valueCellHeight ? textHeight : valueCellHeight;
                        }
                    }

                    rootBand.Columns[column.Key].RowLayoutColumnInfo.PreferredCellSize = new Size(valueCellWidth + 5, valueCellHeight + 5);
                }
            }
        }
    }

    public class Person
    {
        private string mName;
        private string mValue;
        public Person(string name, string value)
        {
            Name = name;
            Value = value;
        }

        public string Name
        {
            get { return mName; }
            set { mName = value; }
        }
        public string Value
        {
            get { return mValue; }
            set { mValue = value; }
        }
    }
}
